        PAGE    60,132
        name    isA20on
        TITLE   isA20on Version 1.6
comment |

Last Updated by Roedy Green 1993 June 8

Purpose
*******

isA20on is a method for diagnosing why some programs fail with a
"packed file corrupt message".


What it Does
************

Very high addresses can be interpreted two ways:

1.  they can reach above the 1 MB barrier into a small region of
    64K A20 memory.  The CPU does this my enabling the A20 address
    line.

2.  they can wrap around to low ram.  The CPU does this by disabling
    the A20 address line.

Normally method 2 is used.  Programs treated with Microsoft's
ExePack require method 2 to be in effect, or you will get a
"packed file corrupt" message.

The method can be controlled by an XMS software driver, such as
HIMEM.SYS or QEMM.  However, extended CMOS determines its initial
setting.

Microsoft unpack routines fail when programs are loaded below
the 64K boundary, because then the A20 ambiguity comes to play.
When there are enough TSRs to force DOS to load above that
bound, you will not notice any problems.

How To Use
**********

Run isA20on.Com

If it says that A20 is off, you are fine as is.

If it says that A20 is on, edit your advanced CMOS settings so
that "Enable CPUA20 to A20" is set to 1 to disable it.  If your
CMOS gives you no way to control the initial setting of the A20
enable, you can use MSFIX.COM to force programs to load above
the 64K boundary.  MSFIX is a kludge.

Method
******

This program is based on a flawed technique used in Microsoft's
HIMEM.asm.  My method is a little more complex, but gets around
the statistical possibility that two different areas of RAM
might accidentally contain the same value.

It simply compares the contents of a high address with a low
address to see if wrap occurred.

Author
******

Please report bugs and problems to:

Roedy Green
Canadian Mind Products
#208 - 525 Ninth Street
New Westminster BC Canada
V5H 2N6
tel:(604) 777-1804
mailto:roedy@mindprod.com
http://mindprod.com

Version 1.0 1990 November 6
 - posted on BIX

Version 1.1 1990 November 6
 - slightly simpler algorithm - compares a single byte

Version 1.2 1990 November 9
 - turns off interrupts longer, just in case other tasks meddle

Version 1.3 1990 November 30
- removed references to "shareware"

Version 1.4 1993 June 8
- embed new address and phone number

Version 1.5 1996 October 25
- embed POB 707 Quathiaski Cove address

Version 1.6 1998 November 8
- embed Barker address
;==============================================================

;       C O N V E N T I O N S

Subroutines may trash any registers they please except CS: DS:
ES: SS: and of course the outputs.

Some of the routines save more registers than this convention
requires.  This is because they were canibalized from other
environments where there were stricter rules.  Speed is not an
issue here.

| ; end of comment

;==============================================================

; M A C R O S

CR      MACRO   ; Carriage return line feed
        DB 0dh,0ah
        ENDM

;======

EOS     MACRO   ; marks end of display string
        DB 0dh,0ah,'$'
        ENDM

;=============================================================
stack   segment stack           ; keep MS link happy by providing null stack
stack   ends

CODE    SEGMENT PARA            ; start off in code.

data    segment byte            ; provide a separate DATA segment
                                ; actually all come after the code

;======================================
;
;       D I S P L A Y   S T R I N G S

CopyrightMsg    label byte
        DB      ' isA20on 1.6 ۲'
        CR
        DB      'Determines the state of the A20 address line.'
        CR
        DB      'Copyright (c) 1990-1999 Roedy Green Canadian Mind Products'
        CR
        DB      '#208 - 525 Ninth Street, New Westminster, BC Canada V3M 5T9',13,10
        DB      'tel:(604) 777-1804   mailto:roedy@mindprod.com   http://mindprod.com',13,10
        DB      'May be freely distributed and used for any purpose except military.'
        CR
        EOS

IsOnMsg         label byte
        DB      ' Warning ۲  A20 address line is enabled.'
        CR
        DB      '                   Microsoft packed Exe files may fail.'
        CR
        DB      '                   See isA20on.Txt for what to do about it.'
        CR
        EOS

IsOffMsg        label byte
        DB      'A20 address line is disabled.  Microsoft packed Exe files will work.'
        CR
        EOS


data            ends

;       P R O C E D U R E S

com     group   code,data       ; force data segment to go at the end!

        ASSUME  CS:com,DS:com,ES:com,SS:com

                                ; seg regs cover everything
        ORG     100H            ; in Code segment
;==============================================================

start:

Main    Proc    near
        lea     dx,CopyRightMsg ; CopyRight banner
        call    Say             ; display
        call    isA20onNow      ; ax=1 if enabled
        jc      A20IsOn

A20IsOff:                       ; A20 is off
        lea     dx,IsOffMsg
        jmp     SayIt

A20IsOn:                        ; A20 is on
        lea     dx,IsOnMsg

SayIt:
        Call    Say

Done:
        mov     ax,4c00h
        int     21h             ;normal termination
main    endP

;==============================================================

;       auxilliary data to help determine A20 status

data    segment         byte

LowMemory   label   dword           ; Set equal to 0000:0080
        dw          00080h
        dw          00000h

HighMemory  label   dword
        dw          00090h          ; Set equal to FFFF:0090
        dw          0FFFFh

data    ends
;==============================================================

isA20onNow      Proc    Near

;  Returns the state of the A20 line
;  returns  carry set if A20 line is enabled, 0 otherwise

;       make compare of low and high RAM TWICE.
;       In between the tests, change low RAM so that false match
;       will be unmasked.

        push    ds
        push    es

        lds     si,cs:LowMemory         ; Compare  byte at 0000:0080
        les     di,cs:HighMemory        ; with the byte at FFFF:0090
        cli                             ; freeze interrupts in case
                                        ; someone else accesses low RAM
        mov     bl,DS:[si]
        mov     bh,ES:[di]
        cmp     bl,bh
        jne     A20On                   ; jump if bytes differ

;       rep do test again, this time with low ram incremented


        inc     byte ptr ds:[si]        ; change low ram
        mov     bl,DS:[si]
        mov     bh,ES:[di]
        dec     byte ptr ds:[si]        ; put low ram back the way it was
        sti                             ; interrupts back on.
        cmp     bl,bh

        je      A20Off                  ; jump if areas same

A20On:
        stc                             ; no wrap, were different
        jmp     goHome

A20Off:
        clc                             ; wrap, were equal
                                        ; Yes, return A20 Disabled
GoHome:
        sti                             ; interrupts back on.
        pop     es
        pop     ds
        ret
isA20onNow  endp

;============================================

Say     Proc    near
;       display message on screen
;       on entry DX contains address of $ delimited message.
        mov     AH,09h
        int     21h
        ret
Say     EndP

;============================================

CODE ends
end start
